'use strict';

var _const = require('../data/const.js');
var bruteforce = require('./guesses/bruteforce.js');
var date = require('./guesses/date.js');
var dictionary = require('./guesses/dictionary.js');
var regex = require('./guesses/regex.js');
var repeat = require('./guesses/repeat.js');
var sequence = require('./guesses/sequence.js');
var spatial = require('./guesses/spatial.js');
var utils = require('./utils.js');

const estimationFunctions = {
  bruteforce: bruteforce,
  dictionary: dictionary,
  spatial: spatial,
  repeat: repeat,
  sequence: sequence,
  regex: regex,
  date: date
};

const getMinGuesses = (match, password) => {
  let minGuesses = 1;

  if (match.token.length < password.length) {
    if (match.token.length === 1) {
      minGuesses = _const.MIN_SUBMATCH_GUESSES_SINGLE_CHAR;
    } else {
      minGuesses = _const.MIN_SUBMATCH_GUESSES_MULTI_CHAR;
    }
  }

  return minGuesses;
}; // ------------------------------------------------------------------------------
// guess estimation -- one function per match pattern ---------------------------
// ------------------------------------------------------------------------------


var estimateGuesses = ((match, password) => {
  const extraData = {}; // a match's guess estimate doesn't change. cache it.

  if ('guesses' in match && match.guesses != null) {
    return match;
  }

  const minGuesses = getMinGuesses(match, password); // @ts-ignore

  const estimationResult = estimationFunctions[match.pattern](match);
  let guesses = 0;

  if (typeof estimationResult === 'number') {
    guesses = estimationResult;
  } else if (match.pattern === 'dictionary') {
    guesses = estimationResult.calculation;
    extraData.baseGuesses = estimationResult.baseGuesses;
    extraData.uppercaseVariations = estimationResult.uppercaseVariations;
    extraData.l33tVariations = estimationResult.l33tVariations;
  }

  const matchGuesses = Math.max(guesses, minGuesses);
  return { ...match,
    ...extraData,
    guesses: matchGuesses,
    guessesLog10: utils.log10(matchGuesses)
  };
});

module.exports = estimateGuesses;
//# sourceMappingURL=estimate.js.map
